@@ -1,22 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1992 Brian Boyter
- * Parts Copyright (c) 1994 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
@@ -25,6 +20,7 @@
 #include "mode.h"
 #include "object.h"
 #include "paintop.h"
+#include "d_line.h"
 #include "u_create.h"
 #include "u_elastic.h"
 #include "u_list.h"
@@ -33,10 +29,11 @@
 
 /*************************** local declarations *********************/
 
-static
-		init_picobj_drawing(), create_picobj(),
-		cancel_picobj();
+static void	init_picobj_drawing();
+static void	create_picobj();
+static void 	cancel_picobj();
 
+void
 picobj_drawing_selected()
 {
     set_mousefun("corner point", "", "", "", "", "");
@@ -49,7 +46,7 @@ picobj_drawing_selected()
     reset_action_on();
 }
 
-static
+static void
 init_picobj_drawing(x, y)
     int		    x, y;
 {
@@ -58,7 +55,7 @@ init_picobj_drawing(x, y)
     canvas_rightbut_proc = cancel_picobj;
 }
 
-static
+static void
 cancel_picobj()
 {
     elastic_box(fix_x, fix_y, cur_x, cur_y);
@@ -66,7 +63,7 @@ cancel_picobj()
     draw_mousefun_canvas();
 }
 
-static
+static void
 create_picobj(x, y)
     int		    x, y;
 {
@@ -117,7 +114,7 @@ create_picobj(x, y)
     box->pic->pix_height = 0;
     box->pic->pix_rotation = 0;
     box->pic->pix_flipped = 0;
-
+    box->pic->figure= (struct f_compound*) NULL;
     box->points = point;
     append_point(fix_x, y, &point);
     append_point(x, y, &point);
