@@ -1,31 +1,27 @@
-#ifndef D_SPLINE_H
-#define D_SPLINE_H
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1985 by Supoj Sutanthavibul
- * Parts Copyright (c) 1994 by Brian V. Smith
- * Parts Copyright (c) 1991 by Paul King
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons who receive
  * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * that this copyright notice remain intact.
+ *
  */
 
+#ifndef D_SPLINE_H
+#define D_SPLINE_H
 
 #define OPEN_SPLINE_MIN_NUM_POINTS     2
 #define CLOSED_SPLINE_MIN_NUM_POINTS   3
 
-extern int         spline_drawing_selected();
-extern int         make_sfactors();
-
+extern void	spline_drawing_selected();
+extern int	make_sfactors();
 
 #endif
