@@ -1,25 +1,22 @@
-#ifndef F_NEUCLRTAB_H
-#define F_NEUCLRTAB_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1994 Anthony Dekker
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
+#ifndef F_NEUCLRTAB_H
+#define F_NEUCLRTAB_H
+
 /*
  * Neural-Net quantization algorithm based on work of Anthony Dekker
  */
@@ -40,7 +37,8 @@
 #define  N_BLU		2
 
 typedef unsigned char  BYTE;	/* 8-bit unsigned integer */
-typedef BYTE  COLR[4];		/* red, green, blue, exponent */
+typedef BYTE	 COLR[4];	/* red, green, blue, exponent */
+extern	BYTE	 clrtab[][3];
 
 #define MIN_NEU_SAMPLES	600	/* min number of samples (npixels/samplefac) needed for network */
 #endif /* F_NEUCLRTAB_H */
