@@ -1,22 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1985 by Supoj Sutanthavibul
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1994 by Brian V. Smith
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
@@ -29,6 +24,7 @@
 #include "fig.h"
 #include "resources.h"
 #include "object.h"
+#include "u_geom.h"
 
 /*************************** COMPUTE NORMAL **********************
 
@@ -153,8 +149,11 @@ compute_arccenter(p1, p2, p3, x, y)
     double	    s12, s13, len1, len2, len3, dx12, dy12, dx13, dy13;
     double	    resx, resy;
 
-    if (p1.x == p3.x && p1.y == p3.y)
-	return 0;
+    if ((p1.x == p3.x && p1.y == p3.y) ||
+	(p1.x == p2.x && p1.y == p2.y) ||
+	(p2.x == p3.x && p2.y == p3.y)) {
+	    return 0;
+    }
 
     dx12 = p1.x - p2.x;
     dy12 = p1.y - p2.y;
