@@ -1,32 +1,29 @@
-#ifndef W_DRAWPRIM_H
-#define W_DRAWPRIM_H
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1985 by Supoj Sutanthavibul
- * Parts Copyright (c) 1994 by Brian V. Smith
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
-extern pr_size	textsize();
-extern PIX_FONT bold_font;
-extern PIX_FONT roman_font;
-extern PIX_FONT button_font;
-extern PIX_FONT canvas_font;
+#ifndef W_DRAWPRIM_H
+#define W_DRAWPRIM_H
+
+extern pr_size      textsize();
+extern XFontStruct *bold_font;
+extern XFontStruct *roman_font;
+extern XFontStruct *button_font;
+extern XFontStruct *canvas_font;
+extern XFontStruct *lookfont();
 
 /* Maximum number of points for polygons etc */
 /* This may be overridden by adding -DMAXNUMPTS=xxxx in the Imakefile/Makefile */
@@ -34,8 +31,6 @@ extern PIX_FONT canvas_font;
 #define		MAXNUMPTS	25000
 #endif /* MAXNUMPTS */
 
-#define MAXLINEWIDTH 500
-
 #define		NORMAL_FONT	"fixed"
 #define		BOLD_FONT	"8x13bold"
 #define		BUTTON_FONT	"6x13"
