@@ -1,22 +1,16 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1985 by Supoj Sutanthavibul
+ * Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1994 by Brian V. Smith
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
@@ -25,23 +19,16 @@
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
+#include "e_edit.h"
 #include "w_dir.h"
 #include "w_drawprim.h"		/* for max_char_height */
 #include "w_export.h"
+#include "w_file.h"
+#include "w_msgpanel.h"
 #include "w_print.h"
 #include "w_setup.h"
 #include "w_util.h"
-
-/* IMPORTS */
-
-extern Boolean	file_msg_is_popped;
-extern Widget	file_msg_popup;
-extern Widget	make_popup_menu();
-extern char    *panel_get_value();
-extern Boolean	query_save();
-extern Widget	file_popup;
-extern Widget	file_dir;
-extern Boolean	popup_up;
+#include "w_icons.h"
 
 /* EXPORTS */
 
@@ -74,6 +61,15 @@ static float    offset_unit_conv[] = { 72.0, 72.0/2.54, 72.0/PIX_PER_INCH };
 /* the bounding box of the figure when the export panel was popped up */
 static int	ux,uy,lx,ly;
 
+/* callback list to keep track of magnification window */
+
+static XtCallbackProc update_mag();
+
+static XtCallbackRec mag_callback[] = {
+	{(XtCallbackProc)update_mag, (XtPointer)NULL},
+	{(XtCallbackProc)NULL, (XtPointer)NULL},
+	};
+
 static String	file_list_translations =
 	"<Btn1Down>,<Btn1Up>: Set()Notify()\n\
 	<Btn1Up>(2): export()\n\
@@ -91,11 +87,6 @@ static void     export_panel_cancel();
 
 static XtCallbackProc update_mag();
 
-static XtCallbackRec mag_callback[] = {
-	{(XtCallbackProc)update_mag, (XtPointer)NULL},
-	{(XtCallbackProc)NULL, (XtPointer)NULL},
-	};
-
 String  exp_translations =
         "<Message>WM_PROTOCOLS: DismissExport()\n";
 
@@ -126,8 +117,7 @@ static Widget	lang_panel, lang_menu, lang_lab;
 static void	transp_select();
 static Widget	transp_lab, transp_menu;
 
-static void	quality_select();
-static Widget	quality_lab, quality_text;
+static Widget	quality_lab, quality_text, quality_spinner;
 
 static void	just_select();
 static Widget	just_menu, just_lab;
@@ -158,11 +148,11 @@ static Widget	exp_yoff_unit_panel, exp_yoff_unit_menu;
 
 static int	xoff_unit_setting, yoff_unit_setting;
 
+DeclareStaticArgs(15);
+
 static void
 export_panel_dismiss()
 {
-    DeclareArgs(2);
-
     /* first get magnification from the widget into appres.magnification
        in case it changed */
     /* the other things like paper size, justification, etc. are already
@@ -175,7 +165,6 @@ export_panel_dismiss()
     FirstArg(XtNstring, "\0");
     SetValues(exp_selfile);		/* clear ascii widget string */
     XtPopdown(export_popup);
-    XtSetSensitive(export_w, True);
     export_up = popup_up = False;
 }
 
@@ -213,10 +202,15 @@ void
 do_export(w)
     Widget	    w;
 {
-	DeclareArgs(2);
 	char	   *fval;
 	int	    xoff, yoff;
 	F_line     *l;
+	char	    transparent[10];
+	int	    transp;
+
+	/* don't export if in the middle of drawing/editing */
+	if (check_action_on())
+		return;
 
 	/* if the popup has been created, get magnification and jpeg quality
 	   here to fix any bad values */
@@ -246,9 +240,13 @@ do_export(w)
 	/* if there is no default export name (e.g. if user has done "New" and not 
 		   entered a name) then make one up */
 	if (!default_export_file || default_export_file[0] == '\0') {
-	    /* for jpeg and tiff use three-letter suffixes */
-	    if (cur_exp_lang==LANG_JPEG)
+	    /* for acad (.sld), jpeg and tiff use three-letter suffixes */
+	    if (cur_exp_lang==LANG_ACAD)
+		sprintf(default_export_file,"NoName.sld");
+#ifdef USE_JPEG
+	    else if (cur_exp_lang==LANG_JPEG)
 		sprintf(default_export_file,"NoName.jpg");
+#endif /* USE_JPEG */
 	    else if (cur_exp_lang==LANG_TIFF)
 		sprintf(default_export_file,"NoName.tif");
 	    else
@@ -289,9 +287,30 @@ do_export(w)
 	app_flush();
 
 	exp_getxyoff(&xoff,&yoff);	/* get x/y offsets from panel */
+	/* convert the transparent color to an RGB hex value */
+	if (cur_exp_lang == LANG_GIF) {
+	    /* use the mapped color for the transparent color */
+	    /* the color was mapped from  Fig.color -> X.color -> mapped.color */
+	    transp = appres.transparent;
+	    /* make background transparent */
+	    if (transp == TRANSP_BACKGROUND) {
+		sprintf(transparent,"#%02x%02x%02x",
+				x_bg_color.red>>8,
+				x_bg_color.green>>8,
+				x_bg_color.blue>>8);
+	    /* make other color transp */
+	    } else {
+		XColor  col;
+		col.pixel = x_color(transp);
+		XQueryColor(tool_d, tool_cm, &col);
+		sprintf(transparent,"#%02x%02x%02x",
+				col.red>>8, col.green>>8, col.blue>>8);
+	    }
+	}
 	/* call fig2dev to export the file */
 	if (print_to_file(fval, lang_items[cur_exp_lang],
-			      appres.magnification, xoff, yoff) == 0) {
+			      appres.magnification, xoff, yoff,
+			      (transp == TRANSP_NONE? NULL: transparent)) == 0) {
 		FirstArg(XtNlabel, fval);
 		SetValues(dfile_text);		/* set the default filename */
 		if (strcmp(fval,default_export_file) != 0)
@@ -320,8 +339,6 @@ just_select(w, new_just, garbage)
     Widget	    w;
     XtPointer	    new_just, garbage;
 {
-    DeclareArgs(2);
-
     FirstArg(XtNlabel, XtName(w));
     SetValues(export_just_panel);
     /* change print justification if it exists */
@@ -335,7 +352,6 @@ papersize_select(w, new_papersize, garbage)
     Widget	    w;
     XtPointer	    new_papersize, garbage;
 {
-    DeclareArgs(2);
     int papersize = (int) new_papersize;
 
     FirstArg(XtNlabel, paper_sizes[papersize].fname);
@@ -351,7 +367,6 @@ multiple_select(w, new_multiple, garbage)
     Widget	    w;
     XtPointer	    new_multiple, garbage;
 {
-    DeclareArgs(2);
     int multiple = (int) new_multiple;
 
     FirstArg(XtNlabel, multiple_pages[multiple]);
@@ -383,13 +398,9 @@ lang_select(w, new_lang, garbage)
     Widget	    w;
     XtPointer	    new_lang, garbage;
 {
-    DeclareArgs(2);
-
     FirstArg(XtNlabel, XtName(w));
     SetValues(lang_panel);
     cur_exp_lang = (int) new_lang;
-    XtSetSensitive(mag_lab, True);
-    XtSetSensitive(export_mag_text, True);
 
     /* enable or disable features available for PostScript or other languages */
 
@@ -453,10 +464,22 @@ transp_select(w, new_transp, garbage)
     Widget	    w;
     XtPointer	    new_transp, garbage;
 {
-    DeclareArgs(1);
+    Pixel	    bgcolor, fgcolor;
+
+    /* get the colors from the color button just pressed */
+    FirstArg(XtNbackground, &bgcolor);
+    NextArg(XtNforeground, &fgcolor);
+    GetValues(w);
+
+    /* get the colorname from the color button and put it and the colors 
+       in the menu button */
     FirstArg(XtNlabel, XtName(w));
+    NextArg(XtNbackground, bgcolor);
+    NextArg(XtNforeground, fgcolor);
     SetValues(export_transp_panel);
     appres.transparent = (int)new_transp;
+
+    XtPopdown(transp_menu);
 }
 
 /* if the export language is JPEG, manage the image quality stuff */
@@ -464,13 +487,17 @@ transp_select(w, new_transp, garbage)
 manage_quality_select()
 {
 	/* if the export language is JPEG, manage the image quality choices */
+#ifdef USE_JPEG
 	if (cur_exp_lang == LANG_JPEG) {
 	    XtManageChild(quality_lab);
-	    XtManageChild(quality_text);
+	    XtManageChild(quality_spinner);
 	} else {
+#endif /* USE_JPEG */
 	    XtUnmanageChild(quality_lab);
-	    XtUnmanageChild(quality_text);
+	    XtUnmanageChild(quality_spinner);
+#ifdef USE_JPEG
 	}
+#endif /* USE_JPEG */
 }
 
 /* if the export language is GIF, manage the transparent color stuff */
@@ -495,7 +522,6 @@ export_update_figure_size()
 	float	mult;
 	char	*unit;
 	char	buf[40];
-	DeclareArgs(2);
 
 	if (!export_popup)
 	    return;
@@ -518,7 +544,6 @@ fit_page()
 	int	lx,ly,ux,uy;
 	float	wd,ht,pwd,pht;
 	char	buf[60];
-	DeclareArgs(2);
 
 	/* get current size of figure */
 	compound_bound(&objects, &lx, &ly, &ux, &uy);
@@ -546,9 +571,12 @@ fit_page()
 	    appres.magnification = 100.0*pwd/wd;
 	else
 	    appres.magnification = 100.0*pht/ht;
+	/* adjust for difference in real metric vs "xfig metric" */
+	if(!appres.INCHES)
+	    appres.magnification *= PIX_PER_CM * 2.54/PIX_PER_INCH;
 
 	/* update the magnification widget */
-	sprintf(buf,"%.2f",appres.magnification);
+	sprintf(buf,"%.1f",appres.magnification);
 	FirstArg(XtNstring, buf);
 	SetValues(export_mag_text);
 
@@ -562,13 +590,12 @@ static void
 get_magnif()
 {
 	char	buf[60];
-	DeclareArgs(2);
 
 	appres.magnification = (float) atof(panel_get_value(export_mag_text));
 	if (appres.magnification <= 0.0)
 	    appres.magnification = 100.0;
 	/* write it back to the widget in case it had a bad value */
-	sprintf(buf,"%.2f",appres.magnification);
+	sprintf(buf,"%.1f",appres.magnification);
 	FirstArg(XtNstring, buf);
 	SetValues(export_mag_text);
 }
@@ -582,7 +609,6 @@ update_mag(widget, item, event)
     int		   *event;
 {
     char	   *buf;
-    DeclareArgs(2);
 
     buf = panel_get_value(export_mag_text);
     appres.magnification = (float) atof(buf);
@@ -593,13 +619,12 @@ static void
 update_figure_size()
 {
     char buf[60];
-    DeclareArgs(2);
 
     export_update_figure_size();
     /* update the print panel's indicators too */
     if (print_popup) {
 	print_update_figure_size();
-	sprintf(buf,"%.2f",appres.magnification);
+	sprintf(buf,"%.1f",appres.magnification);
 	FirstArg(XtNstring, buf);
 	SetValues(print_mag_text);
     }
@@ -611,7 +636,6 @@ static void
 get_quality()
 {
 	char	buf[60];
-	DeclareArgs(2);
 
 	appres.jpeg_quality = (int) atof(panel_get_value(quality_text));
 	if (appres.jpeg_quality <= 0 || appres.jpeg_quality > 100)
@@ -624,25 +648,35 @@ get_quality()
 
 /* create (if necessary) and popup the export panel */
 
+void
 popup_export_panel(w)
     Widget	    w;
 {
-	extern	Atom wm_delete_window;
 	char	buf[60];
-	DeclareArgs(2);
 
 	set_temp_cursor(wait_cursor);
-	XtSetSensitive(w, False);
+	/* if the file panel is up now, get rid of it */
+	if (file_up) {
+		file_up = False;
+		XtPopdown(file_popup);
+	}
+	/* already up? */
+	if (export_up) {
+		/* yes, just raise to top */
+		XRaiseWindow(tool_d, XtWindow(export_popup));
+		return;
+	}
+
 	export_up = popup_up = True;
 
 	if (export_popup) {
-	    /* the export panel has already been created, but the number of 
-	       colors may be different than before.  Re-create the transparent
-	       color menu */
+	    /* the export panel has already been created, but the number of colors
+	       may be different than before.  Re-create the transparent color menu */
 	    XtDestroyWidget(transp_menu);
-	    transp_menu = make_color_popup_menu(export_transp_panel, transp_select, True);
+	    transp_menu = make_color_popup_menu(export_transp_panel,
+	    				"Transparent Color", transp_select, True);
 	    /* also the magnification may have been changed in the print popup */
-	    sprintf(buf,"%.2f",appres.magnification);
+	    sprintf(buf,"%.1f",appres.magnification);
 	    FirstArg(XtNstring, buf);
 	    SetValues(export_mag_text);
 	    /* also the figure size (magnification * bounding_box) */
@@ -671,14 +705,12 @@ popup_export_panel(w)
 	manage_quality_select();
 
 	/* finally, popup the export panel */
-	XtPopup(export_popup, XtGrabNonexclusive);
+	XtPopup(export_popup, XtGrabNone);
 
 	/* insure that the most recent colormap is installed */
 	set_cmap(XtWindow(export_popup));
     	(void) XSetWMProtocols(XtDisplay(export_popup), XtWindow(export_popup),
 			       &wm_delete_window, 1);
-	if (file_msg_is_popped)
-	    XtAddGrab(file_msg_popup, False, False);
 	reset_cursor();
 }
 
@@ -687,7 +719,6 @@ exp_xoff_unit_select(w, new_unit, garbage)
     Widget          w;
     XtPointer       new_unit, garbage;
 {
-    DeclareArgs(2);
     FirstArg(XtNlabel, XtName(w));
     SetValues(exp_xoff_unit_panel);
     xoff_unit_setting = (int) new_unit;
@@ -698,7 +729,6 @@ exp_yoff_unit_select(w, new_unit, garbage)
     Widget          w;
     XtPointer       new_unit, garbage;
 {
-    DeclareArgs(2);
     FirstArg(XtNlabel, XtName(w));
     SetValues(exp_yoff_unit_panel);
     yoff_unit_setting = (int) new_unit;
@@ -707,14 +737,13 @@ exp_yoff_unit_select(w, new_unit, garbage)
 create_export_panel(w)
     Widget	    w;
 {
-	Widget	    	beside, below, exp_off_lab;
-	Widget		entry;
-	PIX_FONT	temp_font;
-	DeclareArgs(12);
-	char		buf[50];
+	Widget	    	 beside, below, exp_off_lab;
+	Widget		 entry,mag_spinner;
+	XFontStruct	*temp_font;
+	char		 buf[50];
 	char		*unit;
-	float		mult;
-	int		i;
+	float		 mult;
+	int		 i;
 
 	export_w = w;
 	XtTranslateCoords(w, (Position) 0, (Position) 0, &xposn, &yposn);
@@ -737,31 +766,40 @@ create_export_panel(w)
 
 	/* Magnification */
 
-	FirstArg(XtNlabel, "  Magnification %");
+	FirstArg(XtNlabel, "     Magnif %");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	mag_lab = XtCreateManagedWidget("mag_label", labelWidgetClass,
 					export_panel, Args, ArgCount);
 
-	FirstArg(XtNwidth, 60);
-	NextArg(XtNfromHoriz, mag_lab);
-	NextArg(XtNeditType, XawtextEdit);
-	sprintf(buf, "%.2f", appres.magnification);
-	NextArg(XtNstring, buf);
-	NextArg(XtNinsertPosition, 3);
-	NextArg(XtNborderWidth, INTERNAL_BW);
+	/* make a spinner entry for the mag */
+	/* note: this was called "magnification" */
+	sprintf(buf, "%.1f", appres.magnification);
 	/* we want to track typing here to update figure size label */
-	NextArg(XtNcallback, mag_callback);
-	export_mag_text = XtCreateManagedWidget("magnification", asciiTextWidgetClass,
-					 export_panel, Args, ArgCount);
+	mag_spinner = MakeFloatSpinnerEntry(export_panel, &export_mag_text, "magnification",
+			(Widget) 0, mag_lab, mag_callback, buf, 0.0, 10000.0);
+	FirstArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	SetValues(mag_spinner);
+
 	XtOverrideTranslations(export_mag_text,
 			   XtParseTranslationTable(export_translations));
 
 	/* Fit Page to the right of the magnification window */
 
 	FirstArg(XtNlabel, "Fit to Page");
-	NextArg(XtNfromHoriz, export_mag_text);
+	NextArg(XtNfromHoriz, mag_spinner);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	fitpage = XtCreateManagedWidget("fitpage", commandWidgetClass,
 				       export_panel, Args, ArgCount);
 	XtAddEventHandler(fitpage, ButtonReleaseMask, (Boolean) 0,
@@ -776,20 +814,31 @@ create_export_panel(w)
 	sprintf(buf, "Fig Size: %.1f%s x %.1f%s      ",
 		(float)(ux-lx)/mult*appres.magnification/100.0,unit,
 		(float)(uy-ly)/mult*appres.magnification/100.0,unit);
+	if ((i = strlen(buf)) < 32)
+	   strcat(buf, "                    "+i-12);
+
 	FirstArg(XtNlabel, buf);
 	NextArg(XtNfromHoriz, fitpage);
 	NextArg(XtNhorizDistance, 5);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	size_lab = XtCreateManagedWidget("size_label", labelWidgetClass,
 					export_panel, Args, ArgCount);
 
 	/* paper size */
 
-	FirstArg(XtNlabel, "       Paper Size");
+	FirstArg(XtNlabel, "   Paper Size");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, fitpage);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	papersize_lab = XtCreateManagedWidget("papersize_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
@@ -798,6 +847,11 @@ create_export_panel(w)
 	NextArg(XtNfromVert, fitpage);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNresizable, True);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	export_papersize_panel = XtCreateManagedWidget("papersize",
 					   menuButtonWidgetClass,
 					   export_panel, Args, ArgCount);
@@ -813,10 +867,14 @@ create_export_panel(w)
 
 	/* Landscape/Portrait Orientation */
 
-	FirstArg(XtNlabel, "      Orientation");
+	FirstArg(XtNlabel, "  Orientation");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, export_papersize_panel);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	orient_lab = XtCreateManagedWidget("orient_label", labelWidgetClass,
 					   export_panel, Args, ArgCount);
 
@@ -824,15 +882,24 @@ create_export_panel(w)
 	NextArg(XtNfromHoriz, orient_lab);
 	NextArg(XtNfromVert, export_papersize_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	export_orient_panel = XtCreateManagedWidget("orientation",
 					     menuButtonWidgetClass,
 					     export_panel, Args, ArgCount);
 	orient_menu = make_popup_menu(orient_items, XtNumber(orient_items),
 				      export_orient_panel, orient_select);
-	FirstArg(XtNlabel, "    Justification");
+	FirstArg(XtNlabel, "Justification");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, export_orient_panel);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	just_lab = XtCreateManagedWidget("just_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
@@ -840,6 +907,11 @@ create_export_panel(w)
 	NextArg(XtNfromHoriz, just_lab);
 	NextArg(XtNfromVert, export_orient_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	export_just_panel = XtCreateManagedWidget("justify",
 					   menuButtonWidgetClass,
 					   export_panel, Args, ArgCount);
@@ -848,10 +920,14 @@ create_export_panel(w)
 
 	/* multiple/single page */
 
-	FirstArg(XtNlabel, "            Pages");
+	FirstArg(XtNlabel, "        Pages");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, export_just_panel);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	multiple_lab = XtCreateManagedWidget("multiple_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
@@ -859,6 +935,11 @@ create_export_panel(w)
 	NextArg(XtNfromHoriz, multiple_lab);
 	NextArg(XtNfromVert, export_just_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	export_multiple_panel = XtCreateManagedWidget("multiple_pages",
 					   menuButtonWidgetClass,
 					   export_panel, Args, ArgCount);
@@ -867,18 +948,24 @@ create_export_panel(w)
 
 	/* X/Y offset choices */
 
-	FirstArg(XtNlabel, "    Export Offset");
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	FirstArg(XtNlabel, "Export Offset");
+	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	exp_off_lab = XtCreateManagedWidget("export_offset_label", labelWidgetClass,
 				     export_panel, Args, ArgCount);
 	FirstArg(XtNlabel, "X");
 	NextArg(XtNfromHoriz, exp_off_lab);
 	NextArg(XtNhorizDistance, 5);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, 1);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	beside = XtCreateManagedWidget("export_offset_lbl_x", labelWidgetClass,
 				     export_panel, Args, ArgCount);
 	FirstArg(XtNwidth, 50);
@@ -886,15 +973,22 @@ create_export_panel(w)
 	NextArg(XtNstring, "0.0");
 	NextArg(XtNinsertPosition, 1);
 	NextArg(XtNfromHoriz, beside);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	export_offset_x = XtCreateManagedWidget("export_offset_x", asciiTextWidgetClass,
 					     export_panel, Args, ArgCount);
 	FirstArg(XtNfromHoriz, export_offset_x);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	exp_xoff_unit_panel = XtCreateManagedWidget(offset_unit_items[appres.INCHES? 0: 1],
 				menuButtonWidgetClass, export_panel, Args, ArgCount);
 	exp_xoff_unit_menu = make_popup_menu(offset_unit_items, XtNumber(offset_unit_items),
@@ -903,9 +997,12 @@ create_export_panel(w)
 	FirstArg(XtNlabel, "Y");
 	NextArg(XtNfromHoriz, exp_xoff_unit_panel);
 	NextArg(XtNhorizDistance, 10);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, 1);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	beside = XtCreateManagedWidget("export_offset_lbl_y", labelWidgetClass,
 				     export_panel, Args, ArgCount);
 	FirstArg(XtNwidth, 50);
@@ -913,15 +1010,22 @@ create_export_panel(w)
 	NextArg(XtNstring, "0.0");
 	NextArg(XtNinsertPosition, 1);
 	NextArg(XtNfromHoriz, beside);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	export_offset_y = XtCreateManagedWidget("export_offset_y", asciiTextWidgetClass,
 					     export_panel, Args, ArgCount);
 	FirstArg(XtNfromHoriz, export_offset_y);
-	NextArg(XtNfromVert, multiple_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, export_multiple_panel);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	exp_yoff_unit_panel = XtCreateManagedWidget(offset_unit_items[appres.INCHES? 0: 1],
 				menuButtonWidgetClass, export_panel, Args, ArgCount);
 	exp_yoff_unit_menu = make_popup_menu(offset_unit_items, XtNumber(offset_unit_items),
@@ -929,19 +1033,26 @@ create_export_panel(w)
 
 	/* The export language is next */
 
-	FirstArg(XtNlabel, "         Language");
+	FirstArg(XtNlabel, "     Language");
 	NextArg(XtNjustify, XtJustifyLeft);
-	NextArg(XtNfromVert, exp_off_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, exp_xoff_unit_panel);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	lang_lab = XtCreateManagedWidget("lang_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, lang_texts[cur_exp_lang]);
 	NextArg(XtNfromHoriz, lang_lab);
-	NextArg(XtNfromVert, exp_off_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, exp_xoff_unit_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	lang_panel = XtCreateManagedWidget("language",
 					   menuButtonWidgetClass,
 					   export_panel, Args, ArgCount);
@@ -955,71 +1066,87 @@ create_export_panel(w)
 
 	FirstArg(XtNlabel, "Transparent color");
 	NextArg(XtNjustify, XtJustifyLeft);
-	NextArg(XtNfromVert, lang_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, lang_panel);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	transp_lab = XtCreateManagedWidget("transp_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
 	set_color_name(appres.transparent, buf);
 	FirstArg(XtNlabel, buf);
 	NextArg(XtNfromHoriz, transp_lab);
-	NextArg(XtNfromVert, lang_lab);
+	NextArg(XtNfromVert, lang_panel);
 	NextArg(XtNwidth, 80);
-	NextArg(XtNvertDistance, 10);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
 	export_transp_panel = XtCreateManagedWidget("transparent",
 					   menuButtonWidgetClass,
 					   export_panel, Args, ArgCount);
-	transp_menu = make_color_popup_menu(export_transp_panel, transp_select, True);
+	transp_menu = make_color_popup_menu(export_transp_panel, 
+					    "Transparent Color", transp_select, True);
 
 	/* and image quality option for JPEG export */
 
 	/* first label */
 	FirstArg(XtNlabel, "Image quality (%)");
 	NextArg(XtNjustify, XtJustifyLeft);
-	NextArg(XtNfromVert, lang_lab);
-	NextArg(XtNvertDistance, 10);
+	NextArg(XtNfromVert, lang_panel);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	quality_lab = XtCreateManagedWidget("quality_label", labelWidgetClass,
 					 export_panel, Args, ArgCount);
 
-	/* text entry for quality */
-	FirstArg(XtNwidth, 40);
-	NextArg(XtNfromHoriz, quality_lab);
-	NextArg(XtNfromVert, lang_lab);
-	NextArg(XtNvertDistance, 10);
-	NextArg(XtNeditType, XawtextEdit);
-	NextArg(XtNborderWidth, INTERNAL_BW);
+	/* spinner entry for quality */
 	sprintf(buf,"%d",appres.jpeg_quality);
-	NextArg(XtNstring, buf);
-	NextArg(XtNinsertPosition, strlen(buf));
-	quality_text = XtCreateManagedWidget("quality_text", asciiTextWidgetClass,
-					    export_panel, Args, ArgCount);
+	quality_spinner = MakeIntSpinnerEntry(export_panel, &quality_text, "quality_text",
+			lang_panel, quality_lab, (XtCallbackRec *) 0, buf, 0, 100);
+	FirstArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	SetValues(quality_spinner);
 
 	/* next the default file name */
 
-	FirstArg(XtNlabel, " Default Filename");
+	FirstArg(XtNlabel, " Default File");
 	NextArg(XtNfromVert, export_transp_panel);
-	NextArg(XtNvertDistance, 15);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	dfile_lab = XtCreateManagedWidget("def_file_label", labelWidgetClass,
 					  export_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, default_export_file);
 	NextArg(XtNfromVert, export_transp_panel);
 	NextArg(XtNfromHoriz, dfile_lab);
-	NextArg(XtNvertDistance, 15);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainRight);
 	dfile_text = XtCreateManagedWidget("def_file_name", labelWidgetClass,
 					   export_panel, Args, ArgCount);
 
-	FirstArg(XtNlabel, "  Output Filename");
+	FirstArg(XtNlabel, "  Output File");
 	NextArg(XtNfromVert, dfile_text);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	nfile_lab = XtCreateManagedWidget("out_file_name", labelWidgetClass,
 					  export_panel, Args, ArgCount);
 
@@ -1035,6 +1162,10 @@ create_export_panel(w)
 	NextArg(XtNstring, named_file);
 	NextArg(XtNinsertPosition, strlen(named_file));
 	NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainRight);
 	exp_selfile = XtCreateManagedWidget("file", asciiTextWidgetClass,
 					    export_panel, Args, ArgCount);
 	XtOverrideTranslations(exp_selfile,
@@ -1047,31 +1178,40 @@ create_export_panel(w)
 	XtOverrideTranslations(exp_selfile,
 			   XtParseTranslationTable(file_name_translations));
 
+	/* create the directory widgets */
 	create_dirinfo(False, export_panel, exp_selfile, &beside, &below,
-		       &exp_mask, &exp_dir, &exp_flist, &exp_dlist);
+		       &exp_mask, &exp_dir, &exp_flist, &exp_dlist, FILE_WIDTH, False);
 	/* make <return> or double click in the file list window export the file */
 	XtOverrideTranslations(exp_flist,
 			   XtParseTranslationTable(file_list_translations));
 
 	FirstArg(XtNlabel, "Cancel");
 	NextArg(XtNfromHoriz, beside);
+	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNfromVert, below);
 	NextArg(XtNvertDistance, 15);
-	NextArg(XtNhorizDistance, 25);
 	NextArg(XtNheight, 25);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainBottom);
+	NextArg(XtNbottom, XtChainBottom);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	cancel_but = XtCreateManagedWidget("cancel", commandWidgetClass,
 					   export_panel, Args, ArgCount);
 	XtAddEventHandler(cancel_but, ButtonReleaseMask, (Boolean) 0,
 			  (XtEventHandler)export_panel_cancel, (XtPointer) NULL);
 
 	FirstArg(XtNlabel, "Export");
-	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, cancel_but);
-	NextArg(XtNvertDistance, 15);
 	NextArg(XtNhorizDistance, 25);
+	NextArg(XtNfromVert, below);
+	NextArg(XtNvertDistance, 15);
 	NextArg(XtNheight, 25);
 	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainBottom);
+	NextArg(XtNbottom, XtChainBottom);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
 	export_but = XtCreateManagedWidget("export", commandWidgetClass,
 					   export_panel, Args, ArgCount);
 	XtAddEventHandler(export_but, ButtonReleaseMask, (Boolean) 0,
@@ -1098,6 +1238,8 @@ create_export_panel(w)
 	    XtSetSensitive(exp_yoff_unit_menu, False);
 	}
 	update_def_filename();
+	/* start the export directory at the current dir */
+	strcpy(export_cur_dir, cur_dir);
 }
 
 /* update the default export filename using the Fig file name */
@@ -1105,7 +1247,6 @@ create_export_panel(w)
 update_def_filename()
 {
     int		    i;
-    DeclareArgs(2);
     char	   *dval;
 
     (void) strcpy(default_export_file, cur_filename);
@@ -1119,9 +1260,13 @@ update_def_filename()
 	    default_export_file[i - 4] = '\0';
 	(void) strcat(default_export_file, ".");
 
-	/* for jpeg and tiff use three-letter suffixes */
-	if (cur_exp_lang==LANG_JPEG)
+	/* for acad (.sld), jpeg and tiff use three-letter suffixes */
+	if (cur_exp_lang==LANG_ACAD)
+	    (void) strcat(default_export_file, "sld");
+#ifdef USE_JPEG
+	else if (cur_exp_lang==LANG_JPEG)
 	    (void) strcat(default_export_file, "jpg");
+#endif /* USE_JPEG */
 	else if (cur_exp_lang==LANG_TIFF)
 	    (void) strcat(default_export_file, "tif");
 	else
@@ -1133,12 +1278,4 @@ update_def_filename()
 	    default_export_file[i] = '\0';
 	else
 	    i = 0;
-    /* set the current directory from the file popup directory */
-    if (file_popup) {
-	FirstArg(XtNstring, &dval);
-	GetValues(file_dir);
-	strcpy(export_cur_dir,dval);
-    } else {
-	strcpy(export_cur_dir,cur_dir);
-    }
 }
