@@ -1,22 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1985 by Supoj Sutanthavibul
+ * Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1998 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1994 by Brian V. Smith
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
@@ -53,47 +48,57 @@ init_grid()
     }
 
     if (appres.INCHES) {
-	posn_rnd[P_MAGNET] = PIX_PER_INCH/16;		/*  1/16" */
+	posn_rnd[P_MAGNET] = PIX_PER_INCH/16;	/*  1/16" */
 	posn_hlf[P_MAGNET] = PIX_PER_INCH/32;
-	posn_rnd[P_GRID1] = PIX_PER_INCH/4;		/*  1/4" */
-	posn_hlf[P_GRID1] = PIX_PER_INCH/8;
-	posn_rnd[P_GRID2] = PIX_PER_INCH/2;		/*  1/2" */
-	posn_hlf[P_GRID2] = PIX_PER_INCH/4;
-	posn_rnd[P_GRID3] = PIX_PER_INCH;		/*  1" */
-	posn_hlf[P_GRID3] = PIX_PER_INCH/2;
-	grid_name[0]      = "1/16 in";			/* only used for points positioning */
-	grid_name[GRID_1] = "1/4 in";
-	grid_name[GRID_2] = "1/2 in";
-	grid_name[GRID_3] = "1 in";
+	posn_rnd[P_GRID1] = PIX_PER_INCH/8;	/*  1/8" */
+	posn_hlf[P_GRID1] = PIX_PER_INCH/16;
+	posn_rnd[P_GRID2] = PIX_PER_INCH/4;	/*  1/4" */
+	posn_hlf[P_GRID2] = PIX_PER_INCH/8;
+	posn_rnd[P_GRID3] = PIX_PER_INCH/2;	/*  1/2" */
+	posn_hlf[P_GRID3] = PIX_PER_INCH/4;
+	posn_rnd[P_GRID4] = PIX_PER_INCH;	/*  1" */
+	posn_hlf[P_GRID4] = PIX_PER_INCH/2;
+	grid_name[GRID_0] = "1/16 inch";	/* only used for points positioning */
+	grid_name[GRID_1] = "1/8 inch";
+	grid_name[GRID_2] = "1/4 inch";
+	grid_name[GRID_3] = "1/2 inch";
+	grid_name[GRID_4] = "1 inch";
 
-	grid_fine[GRID_1] = PIX_PER_INCH/16;	/* 1/16" dot spacing */
-	grid_fine[GRID_2] = PIX_PER_INCH/16;	/* 1/16" dot spacing */
-	grid_fine[GRID_3] = PIX_PER_INCH/8;	/* 1/8" dot spacing */
+	grid_fine[GRID_1] = PIX_PER_INCH/16;	/* 1/16" dot spacing for 1/8 inch grid*/
+	grid_fine[GRID_2] = PIX_PER_INCH/16;	/* 1/16" dot spacing for 1/4 inch grid*/
+	grid_fine[GRID_3] = PIX_PER_INCH/16;	/* 1/16" dot spacing for 1/2 inch grid*/
+	grid_fine[GRID_4] = PIX_PER_INCH/8;	/* 1/8" dot spacing for 1 inch grid*/
 
-	grid_coarse[GRID_1] = PIX_PER_INCH/4;	/* 1/4" grid */
-	grid_coarse[GRID_2] = PIX_PER_INCH/2;	/* 1/2" grid */
-	grid_coarse[GRID_3] = PIX_PER_INCH;	/* 1" grid */
+	grid_coarse[GRID_1] = PIX_PER_INCH/8;	/* 1/8" grid */
+	grid_coarse[GRID_2] = PIX_PER_INCH/4;	/* 1/4" grid */
+	grid_coarse[GRID_3] = PIX_PER_INCH/2;	/* 1/2" grid */
+	grid_coarse[GRID_4] = PIX_PER_INCH;	/* 1" grid */
     } else {
-	posn_rnd[P_MAGNET] = PIX_PER_CM/10;		/* 1 mm */
+	posn_rnd[P_MAGNET] = PIX_PER_CM/10;	/* 1 mm */
 	posn_hlf[P_MAGNET] = PIX_PER_CM/20;
-	posn_rnd[P_GRID1] = PIX_PER_CM/2;		/* 5 mm */
-	posn_hlf[P_GRID1] = PIX_PER_CM/4;
-	posn_rnd[P_GRID2] = PIX_PER_CM;			/* 10 mm */
-	posn_hlf[P_GRID2] = PIX_PER_CM/2;
-	posn_rnd[P_GRID3] = PIX_PER_CM*2;		/* 20 mm */
-	posn_hlf[P_GRID3] = PIX_PER_CM;
-	grid_name[0]      = "1 mm";			/* only used for points positioning */
-	grid_name[GRID_1] = "5 mm";
-	grid_name[GRID_2] = "10 mm";
-	grid_name[GRID_3] = "20 mm";
+	posn_rnd[P_GRID1] = PIX_PER_CM/5;	/* 2 mm */
+	posn_hlf[P_GRID1] = PIX_PER_CM/10;
+	posn_rnd[P_GRID2] = PIX_PER_CM/2;	/* 5 mm */
+	posn_hlf[P_GRID2] = PIX_PER_CM/4;
+	posn_rnd[P_GRID3] = PIX_PER_CM;		/* 10 mm */
+	posn_hlf[P_GRID3] = PIX_PER_CM/2;
+	posn_rnd[P_GRID4] = PIX_PER_CM*2;	/* 20 mm */
+	posn_hlf[P_GRID4] = PIX_PER_CM;
+	grid_name[GRID_0] = "1 mm";		/* only used for points positioning */
+	grid_name[GRID_1] = "2 mm";
+	grid_name[GRID_2] = "5 mm";
+	grid_name[GRID_3] = "10 mm";
+	grid_name[GRID_4] = "20 mm";
 
-	grid_fine[GRID_1] = PIX_PER_CM/10;	/*  1 mm dot spacing */
-	grid_fine[GRID_2] = PIX_PER_CM/5;	/*  2 mm dot spacing */
-	grid_fine[GRID_3] = PIX_PER_CM/2;	/*  5 mm dot spacing */
+	grid_fine[GRID_1] = PIX_PER_CM/10;	/*  1 mm dot spacing for 5 mm grid */
+	grid_fine[GRID_2] = PIX_PER_CM/10;	/*  1 mm dot spacing for 5 mm grid */
+	grid_fine[GRID_3] = PIX_PER_CM/5;	/*  2 mm dot spacing for 10 mm grid */
+	grid_fine[GRID_4] = PIX_PER_CM/2;	/*  5 mm dot spacing for 20 mm grid */
 
-	grid_coarse[GRID_1] = PIX_PER_CM/2;	/*  5 mm grid */
-	grid_coarse[GRID_2] = PIX_PER_CM;	/* 10 mm grid */
-	grid_coarse[GRID_3] = PIX_PER_CM*2;	/* 20 mm grid */
+	grid_coarse[GRID_1] = PIX_PER_CM/5;	/*  2 mm grid */
+	grid_coarse[GRID_2] = PIX_PER_CM/2;	/*  5 mm grid */
+	grid_coarse[GRID_3] = PIX_PER_CM;	/* 10 mm grid */
+	grid_coarse[GRID_4] = PIX_PER_CM*2;	/* 20 mm grid */
     }
 }
 
@@ -118,7 +123,7 @@ setup_grid(grid)
 	else
 	    fine = grid_fine[grid] / ZOOM_FACTOR;
 	/* if zoom is small, use larger grid */
-	while (coarse < 8 && ++grid <= GRID_3) {
+	while (coarse < 6 && ++grid <= GRID_4) {
 	    coarse = grid_coarse[grid] * zoomscale;
 	    fine = grid_fine[grid] * zoomscale;
 	}
@@ -127,9 +132,11 @@ setup_grid(grid)
 	    coarse = fine;
 	    fine = 0.0;
 	}
-	/* size of the pixmap equal to 1 inch or 2 cm to reset any error at those boundaries */
-	dim = round(appres.INCHES? PIX_PER_INCH*zoomscale: 2*PIX_PER_CM*zoomscale);
-
+	/* size of the pixmap equal to 1 inch or 2 cm to reset any 
+	   error at those boundaries */
+	dim = round(appres.INCHES? 
+			PIX_PER_INCH*zoomscale:
+			2*PIX_PER_CM*zoomscale);
 	if (grid_pm)
 	    XFreePixmap(tool_d, grid_pm);
 	grid_pm = XCreatePixmap(tool_d, canvas_win, dim, dim, tool_dpth);
